package command

import (
	"context"

	"github.com/pkg/errors"

	"forge.cadoles.com/wpetit/fake-smtp/internal/model"
	"forge.cadoles.com/wpetit/fake-smtp/internal/storm"
	"gitlab.com/wpetit/goweb/cqrs"
	"gitlab.com/wpetit/goweb/middleware/container"
)

type ClearInboxRequest struct{}

func HandleClearInbox(ctx context.Context, cmd cqrs.Command) error {
	_, ok := cmd.Request().(*ClearInboxRequest)
	if !ok {
		return cqrs.ErrUnexpectedRequest
	}

	ctn, err := container.From(ctx)
	if err != nil {
		return errors.Wrap(err, "could not retrieve service container")
	}

	db, err := storm.From(ctn)
	if err != nil {
		return errors.Wrap(err, "could not retrieve storm service")
	}

	if err := db.Select().Delete(&model.Email{}); err != nil {
		if err == storm.ErrNotFound {
			return nil
		}

		return errors.Wrap(err, "could not delete emails")
	}

	return nil
}
